local E, L, V, P, G = unpack(ElvUI)
local mPlugin = "mMediaTag"
local mMT = E:GetModule(mPlugin)
local addon, ns = ...

local _G = _G
local next, type = next, type
local gmatch, gsub, format = gmatch, gsub, format
local unpack, pairs, wipe, floor, ceil = unpack, pairs, wipe, floor, ceil
local strfind, strmatch, strlower, strsplit = strfind, strmatch, strlower, strsplit
local utf8lower, utf8sub, utf8len = string.utf8lower, string.utf8sub, string.utf8len


local function mAbbrev(text)
	local letters, lastWord = '', strmatch(text, '.+%s(.+)$')
	if lastWord then
		for word in gmatch(text, '.-%s') do
			local firstLetter = utf8sub(gsub(word, '^[%s%p]*', ''), 1, 1)
			if firstLetter ~= utf8lower(firstLetter) then
				letters = format('%s%s. ', letters, firstLetter)
			end
		end
		text = format('%s%s', letters, lastWord)
	end
	return text
end

--Color for various Functions
function mMT:mColorDatatext()
    local nhc, hc, myth, mythp, other, titel, tip = E.db[mPlugin].mDataText.colornhc.hex, E.db[mPlugin].mDataText.colorhc.hex, E.db[mPlugin].mDataText.colormyth.hex, E.db[mPlugin].mDataText.colormythplus.hex, E.db[mPlugin].mDataText.colorother.hex, E.db[mPlugin].mDataText.colortitel.hex, E.db[mPlugin].mDataText.colortip.hex
    return nhc, hc, myth, mythp, other, titel, tip
end

--Great Vault Functions
local function mColorGardient(level)
    local r, g, b = E:ColorGradient(level * 0.003 ,  0, .43, .86, .63, .2, .93, .89, .16, .31)
    return E:RGBToHex(r, g, b)
end

local vaultinfo = {}
local vaultinforaid, vaultinfomplus, vaultinfopvp, vaultinfohighest, ok = {}, {}, {}

function mMT:mGetVaultInfo()
    wipe(vaultinfo); wipe(vaultinforaid); wipe(vaultinfomplus); wipe(vaultinfopvp);
    vaultinfo = C_WeeklyRewards.GetActivities()
    if not vaultinfo then 
        return {}, {}, {}, nil, false
    else
        local vaultinfohighest, ok = 0, false
        for i = 1, 9 do
            local id = vaultinfo[i] and vaultinfo[i].id or 0
            local itemLink = C_WeeklyRewards.GetExampleRewardItemHyperlinks(id)
            if itemLink then
                local ItemLevelInfo = GetDetailedItemLevelInfo(itemLink)
                if ItemLevelInfo then
                    vaultinfohighest = (vaultinfohighest and (vaultinfohighest < ItemLevelInfo) and ItemLevelInfo or vaultinfohighest) or (not vaultinfohighest and ItemLevelInfo)
                    if i < 4 then
                        table.insert(vaultinfomplus, format("%s%s|r", mColorGardient(ItemLevelInfo), ItemLevelInfo))
                    elseif i < 7 then
                        table.insert(vaultinforaid, format("%s%s|r", mColorGardient(ItemLevelInfo), ItemLevelInfo))
                    else
                        table.insert(vaultinfopvp, format("%s%s|r", mColorGardient(ItemLevelInfo), ItemLevelInfo))
                    end
                    ok = true
                end
            end
        end
        vaultinfohighest = format("%s%s|r", mColorGardient(tonumber(vaultinfohighest)), vaultinfohighest)
        return vaultinforaid, vaultinfomplus, vaultinfopvp, vaultinfohighest, ok
    end
end

--Dungeon Difficulty
function mMT:DungeonDifficultyShort()
    local name, instanceType, instanceDifficultyID, difficultyName, maxPlayers, dynamicDifficulty, isDynamicInstance = GetInstanceInfo()
    local nhc, hc, myth, mythp, other, titel = mMT:mColorDatatext()

    if instanceDifficultyID == 1 or instanceDifficultyID == 3 or instanceDifficultyID == 4 or instanceDifficultyID == 14 then
        return format("%s%s|r", nhc, L["N"])
    elseif instanceDifficultyID == 2 or instanceDifficultyID == 5 or instanceDifficultyID == 6 or instanceDifficultyID == 15 or instanceDifficultyID == 39 or instanceDifficultyID == 149 then
        return format("%s%s|r", hc, L["H"])
    elseif instanceDifficultyID == 23 or instanceDifficultyID == 16 or instanceDifficultyID == 40 then
        return format("%s%s|r", myth, L["M"])
    elseif instanceDifficultyID == 8 then
        local keyStoneLevel, _ = C_ChallengeMode.GetActiveKeystoneInfo()
        local r, g, b = E:ColorGradient(keyStoneLevel * 0.06 , .1, 1, .1, 1, 1, .1, 1, .1, .1)
        if keyStoneLevel ~= nil and C_MythicPlus.IsMythicPlusActive() and (C_ChallengeMode.GetActiveChallengeMapID() ~= nil) then
            return format(L["%sM|r%s+%s|r"], mythp, E:RGBToHex(r, g, b), keyStoneLevel)
        else
            return format("%s%s|r", mythp, L["M+"])
        end
    else
        return format("%s%s|r", other, E:ShortenString(difficultyName, 1))
    end
end

function mMT:MythPlusDifficultyShort()
    local name, instanceType, instanceDifficultyID, difficultyName, maxPlayers, dynamicDifficulty, isDynamicInstance = GetInstanceInfo()
    local nhc, hc, myth, mythp, other, titel = mMT:mColorDatatext()

    if instanceDifficultyID == 8 then
        local keyStoneLevel, _ = C_ChallengeMode.GetActiveKeystoneInfo()
        local r, g, b = E:ColorGradient(keyStoneLevel * 0.06 , .1, 1, .1, 1, 1, .1, 1, .1, .1)
        if keyStoneLevel ~= nil and C_MythicPlus.IsMythicPlusActive() and (C_ChallengeMode.GetActiveChallengeMapID() ~= nil) then
            return format("%s%s|r", E:RGBToHex(r, g, b), keyStoneLevel)
        else
            return format("%s%s|r", mythp, L["M+"])
        end
    else
        return format("%s%s|r", other, E:ShortenString(difficultyName, 1))
    end
end

--Keystone Functions
local OwenKeystoneText = {}
function mMT:OwenKeystone()
    wipe(OwenKeystoneText)
    local keyStoneLevel, keyStoneLevelMax = C_MythicPlus.GetOwnedKeystoneLevel(), 20

    if keyStoneLevel then
        local challengeMapID = C_MythicPlus.GetOwnedKeystoneChallengeMapID()
        local name, id, timeLimit, texture, backgroundTexture = C_ChallengeMode.GetMapUIInfo(challengeMapID)
        local r, g, b = E:ColorGradient(keyStoneLevel * 0.06 , .1, 1, .1, 1, 1, .1, 1, .1, .1) 
        local nhc, hc, myth, _, other, titel = mMT:mColorDatatext()

        if keyStoneLevel then
            table.insert(OwenKeystoneText, 1, format("%s%s|r", titel, L["Mythic Plus Keystone"]))
            table.insert(OwenKeystoneText, 2, format("%s%s: |r %s%s|r%s +%s|r", other, L["Keystone"], myth, name, E:RGBToHex(r, g, b), keyStoneLevel))
        end

        return OwenKeystoneText
    end
end

--Affixes Functions
local WeeklyAffixesText = {}
function mMT:WeeklyAffixes()
    wipe(WeeklyAffixesText)
    local nhc, hc, myth, _, other, titel = mMT:mColorDatatext()
    local AffixText = nil

	if (date("%u") == "2") or (date("%u") == "3") and not E.db[mPlugin].mSavedAffixes.reset then
		E.db[mPlugin].mSavedAffixes.affixes = nil
		E.db[mPlugin].mSavedAffixes.reset = true
	elseif (date("%u") ~= "2") and (date("%u") ~= "3") then
		E.db[mPlugin].mSavedAffixes.reset = false
	end

	if (date("%u") ~= "2") and (date("%u") ~= "3") and (tonumber(E.db[mPlugin].mSavedAffixes.day) >= (tonumber(date("%j")) +5)) then
		affixes = E.db[mPlugin].mSavedAffixes.affixes
		if (affixes) == nil then
			affixes = C_MythicPlus.GetCurrentAffixes()
			if not (affixes) == nil then
				E.db[mPlugin].mSavedAffixes.affixes = affixes
			end
		end
	else
		affixes = C_MythicPlus.GetCurrentAffixes()
		E.db[mPlugin].mSavedAffixes.affixes = affixes
		E.db[mPlugin].mSavedAffixes.day = date("%j")
	end

	if (affixes) then
		for i, affix in ipairs(affixes) do
			local name, _, _ = C_ChallengeMode.GetAffixInfo(affix.id)
			if AffixText == nil then
				AffixText = name
			else
				AffixText = format("%s, %s", AffixText, name)
			end
		end
	else
		affixes = C_MythicPlus.GetCurrentAffixes()
		if not (affixes) == nil then
			E.db[mPlugin].mSavedAffixes.affixes = affixes
		end
	end

	if AffixText ~= nil then
		local seasonID = C_MythicPlus.GetCurrentSeason()
		if seasonID <= 0 then
			seasonID = E.db[mPlugin].mSavedAffixes.season
		end
		
		if seasonID >= 1 then
            E.db[mPlugin].mSavedAffixes.season = seasonID
            
            table.insert(WeeklyAffixesText, 1, format("%s%s|r", titel,  L["This Week Affix"]))
            table.insert(WeeklyAffixesText, 2, format("%s%s|r", other, AffixText))
        
            return WeeklyAffixesText
        else
            table.insert(WeeklyAffixesText, 3, format("%s%s|n%s|r", ns.mColor5, L["Saisson has not started yet."], L["%sERROR! Please open the Mythic+ window, LFG Tool or Reload UI!|r"]))
			return WeeklyAffixesText
		end
    else
        table.insert(WeeklyAffixesText, 3, format(L["%sERROR! Please open the Mythic+ window, LFG Tool or Reload UI!|r"], ns.mColor5))
		return WeeklyAffixesText
	end
end

--Instance Settings Player
local InctanceInfoText = {}
function mMT:InctanceInfo()
    wipe(InctanceInfoText)
    local DungeonID = GetDungeonDifficultyID()
	local RaidID = GetRaidDifficultyID()
	local DungeonName, _, _, _, _, _, _ = GetDifficultyInfo(DungeonID)
	local RaidName, _, _, _, _, _, _ = GetDifficultyInfo(RaidID)
    local nhc, hc, myth, _, other, titel = mMT:mColorDatatext()

    table.insert(InctanceInfoText, 1, format("%s%s|r", titel, L["Instancoptions"]))

    if DungeonID == 1 then
        table.insert(InctanceInfoText, 2, format("%s%s:|r %s%s|r", other, DUNGEON_DIFFICULTY, nhc, DungeonName))
    elseif DungeonID == 2 then
        table.insert(InctanceInfoText, 2, format("%s%s:|r %s%s|r", other, DUNGEON_DIFFICULTY, hc, DungeonName))
    elseif DungeonID == 23 then
        table.insert(InctanceInfoText, 2, format("%s%s:|r %s%s|r", other, DUNGEON_DIFFICULTY, myth, DungeonName))
    else
        table.insert(InctanceInfoText, 2, format("%s%s:|r %s%s|r", other, DUNGEON_DIFFICULTY, other, DungeonName))
    end

    if RaidID == 14 then
        table.insert(InctanceInfoText, 3, format("%s%s:|r %s%s|r", other, RAID_DIFFICULTY, nhc, RaidName))
    elseif RaidID == 15 then
        table.insert(InctanceInfoText, 3, format("%s%s:|r %s%s|r", other, RAID_DIFFICULTY, hc, RaidName))
    elseif RaidID == 16 then
        table.insert(InctanceInfoText, 3, format("%s%s:|r %s%s|r", other, RAID_DIFFICULTY, myth, RaidName))
    else
        table.insert(InctanceInfoText, 3, format("%s%s:|r %s%s|r", other, RAID_DIFFICULTY, other, RaidName))
    end

    return InctanceInfoText
end

--Instance Informations Dungeon
local DungeonInfoText ={}
function mMT:DungeonInfo()
    wipe(DungeonInfoText)
    local nhc, hc, myth, mythp, other, titel, tip = mMT:mColorDatatext()
	local name, instanceType, instanceDifficultyID, difficultyName, maxPlayers, dynamicDifficulty, isDynamicInstance = GetInstanceInfo()

    table.insert(DungeonInfoText, 1, format("%s%s|r", titel, L["Instance"]))
	table.insert(DungeonInfoText, 2, format("%s%s:|r %s%s|r", other, L["Name"], other, name))


    if instanceDifficultyID == 1 or instanceDifficultyID == 3 or instanceDifficultyID == 4 or instanceDifficultyID == 14 then
        table.insert(DungeonInfoText, 3, format("%s%s:|r %s%s|r (%s)", other, L["Difficulty"], nhc, difficultyName, maxPlayers))
        table.insert(DungeonInfoText, 4, format("%s%s|r", nhc, mAbbrev(name)))
    elseif instanceDifficultyID == 2 or instanceDifficultyID == 5 or instanceDifficultyID == 6 or instanceDifficultyID == 15 or instanceDifficultyID == 39 or instanceDifficultyID == 149 then
        table.insert(DungeonInfoText, 3, format("%s%s:|r %s%s|r (%s)", other, L["Difficulty"], hc, difficultyName, maxPlayers))
        table.insert(DungeonInfoText, 4, format("%s%s|r", hc, mAbbrev(name)))
    elseif instanceDifficultyID == 23 or instanceDifficultyID == 16 or instanceDifficultyID == 40 then
        table.insert(DungeonInfoText, 3, format("%s%s:|r %s%s|r (%s)", other, L["Difficulty"], myth, difficultyName, maxPlayers))
        table.insert(DungeonInfoText, 4, format("%s%s|r", myth, mAbbrev(name)))
    elseif instanceDifficultyID == 8 then
        table.insert(DungeonInfoText, 3, format("%s%s:|r %s%s|r (%s)", other, L["Difficulty"], mythp, difficultyName, maxPlayers))
        table.insert(DungeonInfoText, 4, format("%s%s|r", mythp, mAbbrev(name)))
    else
        table.insert(DungeonInfoText, 3, format("%s%s:|r %s%s|r (%s)", other, L["Difficulty"], other, difficultyName, maxPlayers))
        table.insert(DungeonInfoText, 4, format("%s%s|r", other, mAbbrev(name)))
    end

    return DungeonInfoText
end

function mMT:DungeonInfoName()
    local nhc, hc, myth, mythp, other, titel, tip = mMT:mColorDatatext()
	local name, instanceType, instanceDifficultyID, difficultyName, maxPlayers, dynamicDifficulty, isDynamicInstance = GetInstanceInfo()

    if instanceDifficultyID == 1 or instanceDifficultyID == 3 or instanceDifficultyID == 4 or instanceDifficultyID == 14 then
        return format("%s%s|r", nhc, mAbbrev(name))
    elseif instanceDifficultyID == 2 or instanceDifficultyID == 5 or instanceDifficultyID == 6 or instanceDifficultyID == 15 or instanceDifficultyID == 39 or instanceDifficultyID == 149 then
        return format("%s%s|r", hc, mAbbrev(name))
    elseif instanceDifficultyID == 23 or instanceDifficultyID == 16 or instanceDifficultyID == 40 then
        return format("%s%s|r", myth, mAbbrev(name))
    elseif instanceDifficultyID == 8 then
        return format("%s%s|r", mythp, mAbbrev(name))
    else
        return format("%s%s|r", other, mAbbrev(name))
    end
end

--Instance Informations Myth+
local MythicPlusDungeonText ={}
function mMT:MythicPlusDungeon()
    wipe(MythicPlusDungeonText)
	local keyStoneLevel, _ = C_ChallengeMode.GetActiveKeystoneInfo()
    local name, instanceType, instanceDifficultyID, difficultyName, maxPlayers, dynamicDifficulty, isDynamicInstance = GetInstanceInfo()
    local nhc, hc, myth, mythp, other, titel, tip = mMT:mColorDatatext()
    local r, g, b = E:ColorGradient(keyStoneLevel * 0.06 , .1, 1, .1, 1, 1, .1, 1, .1, .1)

    table.insert(MythicPlusDungeonText, 1, format("%s%s|r", titel, L["Mythic Plus"]))
    table.insert(MythicPlusDungeonText, 2, format("%s%s: |r %s%s|r%s +%s|r", other, L["Keystone"], myth, name, E:RGBToHex(r, g, b), keyStoneLevel))

    return MythicPlusDungeonText
end

--Achievements
local mAchievementInfoText = {}
local function mAchievementInfo(mID)
    wipe(mAchievementInfoText)
	local id, name, points, completed, month, day, year, description, flags, icon, rewardText, isGuild, wasEarnedByMe, earnedBy = GetAchievementInfo(mID)
	mAchievementInfoText = {name = name, icon = mMT:mIcon(icon)}
	return mAchievementInfoText
end

local AchievementText = {}
function mMT:Achievement(option)
    wipe(AchievementText)
    local _, _, _, _, other, titel = mMT:mColorDatatext()
	if option == 15 then 
        local info0 = mAchievementInfo(14532)
        table.insert(AchievementText, 0, format("%s%s|r %s", titel, info0.name, info0.icon))
		for crit=1,8 do 
			local a,_,c = GetAchievementCriteriaInfo(14532, crit~=9 and crit or 10)
			table.insert(AchievementText, crit, {format("%s%s|r", other, a), string.format("[%s]", c and format("%s%s|r", ns.mColor6, L["True"]) or format("%s%s|r", ns.mColor5, L["False"]))})
		end
	elseif option == 10 then
        local info0 = mAchievementInfo(14531)
        table.insert(AchievementText, 0, format("%s%s|r %s", titel, info0.name, info0.icon))
		for crit=1,8 do
			local a,_,c = GetAchievementCriteriaInfo(14531, crit~=9 and crit or 10)
			table.insert(AchievementText, crit, {format("%s%s|r", other, a), string.format("[%s]", c and format("%s%s|r", ns.mColor6, L["True"]) or format("%s%s|r", ns.mColor5, L["False"]))})
		end
	elseif option == 0 then
        local info0 = mAchievementInfo(14418)
        table.insert(AchievementText, 0, format("%s%s|r %s", titel, info0.name, info0.icon))
		for crit=1,8 do
			local a,_,c = GetAchievementCriteriaInfo(14418, crit~=9 and crit or 10)
			table.insert(AchievementText, crit, {format("%s%s|r", other, a), string.format("[%s]", c and format("%s%s|r", ns.mColor6, L["True"]) or format("%s%s|r", ns.mColor5, L["False"]))})
		end
	end
    return AchievementText
end